#!/bin/bash

workdir="$(mktemp -d playlist-XXXXXX)"
ads_dir="../assets/ads/"
today="$(date +"%y/%m/%d")"
res_dir="radio/${today}"
current_player=""
job_started=0
mkdir -p "$res_dir"

function player(){
  if [[ -n "$current_player" ]];then
    res="$(eval "playerctl --player=$current_player $@ || echo 'ERROR'")" 
  else
    res="$(eval "playerctl $@ || echo 'ERROR'")"
  fi
  echo "$res"
}

function cleanup_and_exit() {
  if [[ "$job_started" = "1" ]];then
    rm -fv "current.mp3" "bg.png" "cover.jpg" "cover.png"
    ffmpeg -f concat -i concat -c copy "compiled-$((compiled_index + 1)).webm"
    rm -fv "concat"
    popd
    ls "$workdir" | grep compiled | xargs -I '{}' mv -v $workdir/'{}' "$res_dir"
  fi
  rm -rfv "$workdir"
}

function read_player(){
  echo "Which player should I use?"
  printf "Available players:\n$(playerctl -l | sed "s/^/* /")\n"
  read -p '> ' current_player
}

trap "cleanup_and_exit" EXIT

if [[ !( $(playerctl -l | wc -l) -eq 1) ]];then
  read_player
else
  current_player="$(playerctl -l)"
fi

if [[ !( $(player status) ) ]]; then
  echo "Can't find any player"
  exit 1
fi

if [[ $(player status) != "Playing" ]]; then
  player next
fi

if [[ -e "${res_dir}" ]];then
  compiled_index="$(ls "$res_dir" | grep compiled | sort --numeric-sort | tail -n 1 | cut -d '.' -f1 | cut -d '-' -f2)"
else
  compiled_index="-1"
fi
index="0"

pushd "$workdir"

job_started=1

while [[ "$(player status)" = "Playing" ]];do
  title="$(player metadata xesam:title)"
  artist="$(player metadata xesam:artist)"
  artUrl="$(player metadata mpris:artUrl)"
  url="$(player metadata xesam:url)"

  # Downloading song and album cover
  echo "Loading song"
  wget -q "${url}" -O "current.mp3"
  echo "loaded"

  echo "Loading album cover"
  wget -q "${artUrl}" -O "cover.jpg"
  echo "loaded"

  # Processing background
  music_bg test "cover.jpg" --output "cover.png"
  
  # Adding song name
  convert -background '#0008' -gravity center \
    -fill white -size 800x70 caption:"${artist} - ${title}" \
    "cover.png" +swap -gravity south -geometry +0+200 -composite "bg.png"
  
  rm -fv "cover.jpg" "cover.png"

  target_adv="$(find "$ads_dir" -type f | shuf -n 1)"
  echo "Rendering ${index}"
  ffpb -loop 1 -r 3 -i bg.png -i current.mp3 -r 25 -i "${target_adv}" \
    -filter_complex "[0:v]scale=w=1280:h=720:force_original_aspect_ratio=2,crop=1280:720[img]; \
    [2:v]setpts=10*PTS[adv]; \
    [adv]loop[ladv]; \
    [img][ladv]overlay=enable='gt(mod(t, 40), 30)'[rv]" \
    -map "[rv]" \
    -map 1:a \
    -framerate 30 -g 60 \
    -c:v vp8 -pix_fmt yuv420p -movflags +faststart \
    -vb 4000k \
    -f webm \
    -c:a libopus \
    -err_detect ignore_err \
    -shortest -strict -2 -y \
    "${index}.webm"
  
  echo "file '${index}.webm'" >> concat

  index=$((index + 1))
  if [[ "$(player next)" = "ERROR" ]];then
    break
  fi
done

player stop
